unit LINKP;

interface

procedure PICCursorOFF;
procedure PICCursorON;
procedure PICReadSettings(section:string);
procedure PICDefineChar(cn,d1,d2,d3,d4,d5,d6,d7,d8:byte);
procedure PICpushchar(ch:byte);
procedure PICpushins(ch:byte);
procedure PICClose;
procedure PICInit;
procedure PICPrintLCD(msg:string);
procedure PICClearLCD;
procedure PICGotoXY(row,col:byte);

implementation
uses Dos, Crt,config;

{$F+}
{ this stuff is for the communications part }
const
    { registers in UART }
    SER_RBF =  0;   { the read buffer }
    SER_THR =  0;   { the write buffer }
    SER_IER =  1;   { the int. enable register }
    SER_IIR =  2;   { the int. identification register }
    SER_LCR =  3;   { control data config. and divisor latch }
    SER_MCR =  4;   { modem control reg. }
    SER_LSR =  5;   { line status reg. }
    SER_MSR =  6;   { modem status of cts, ring etc. }
    SER_DLL =  0;   { the low byte of baud rate divisor }
    SER_DLH =  1;   { the hi byte of divisor latch }

   { COM2    = $3F8; { COM2 port number }
    ua         : array[1..4] of word = ($3f8,$2f8,$3e8,$2e8);

type
  AnArray   = array[0..1024] of byte;

var
  old_isr : pointer;                { holds old com port interrupt handler }
  ser_buffer : AnArray;             { the receive buffer}
  ser_end,ser_start : integer;      { indexes into receive buffer }
  old_int_mask : byte;              { current character }
  serial_lock : byte;
  out_buf : string[24];             { isn't altered while it is being written }
  chb : char;
  ch1 : word;
i:integer;
  comport,baudrate:word;

function inport(portnum:integer): byte;
  var
    alreg : byte;
  begin
    asm
      cli
      mov  dx,portnum
      in   al,dx
      mov  alreg,al
      sti
    end;
    inport := alreg;
  end;

procedure outport(portnum:word; val:byte);
  begin
    asm
      cli
      mov  dx,portnum
      mov  al,val
      out  dx,al
      sti
    end;
  end;

procedure serial_isr; interrupt;
  var
    ser_ch : byte;
  begin
    serial_lock := 1;    { lock out other functions so buffer doesn't get corrupted }
    ser_ch := inport(ua[comport] + SER_RBF);   { place character into next position }
    ser_end := ser_end + 1;
    if (ser_end > 1023) then            { wrap buffer index around }
      ser_end := 0;
    ser_buffer[ser_end] := ser_ch;      { move character into buffer }
    outport($20, $20);                  { restore PIC }
    serial_lock := 0;                   { undo lock }
  end;

{ this function reads a character from the circulating buffer }
{ and returns it to the caller }
function serial_read : word;
  var
    ch : byte;
  begin
    repeat
    until serial_lock = 0;     { wait for isr to end }

{ is there a character(s) ready in buffer? }
    if (ser_end <> ser_start) then
      begin
        ser_start := ser_start + 1;
        if (ser_start > 1023) then { wrap buffer index if needed }
          ser_start := 0;
        ch := ser_buffer[ser_start];  { get the character out of buffer }
        serial_read := word(ch);   { send data back to caller }
      end
    else
      serial_read := $FFFF;  { buffer was empty return -1 }
  end;


{ this function writes a character to the transmit buffer, but first it }
{ waits for the transmit buffer to be empty. }
procedure serial_write(ch:byte);
  begin
{ wait for transmit buffer to be empty }
    repeat
    until (inport(ua[comport] + SER_LSR) AND $20) <> 0;

    outport(ua[comport] + SER_THR, byte(ch));    { send the character }
  end;

procedure open_serial;
  var
    ch : byte;
  begin
    asm
      cli
    end;

    outport(ua[comport] + SER_LCR, $80);  { set the baud rate }
    outport(ua[comport] + SER_DLL, 12);
    outport(ua[comport] + SER_DLH, 0);
    outport(ua[comport] + SER_LCR, $03);  { set the configuration for the port }
    ch := inport(ua[comport] + SER_RBF);  { do a read just incase one already there }

{ enable the interrupt when char is ready to be received }
    outport(ua[comport] + SER_MCR, $0B);
    outport(ua[comport] + SER_IER, 1);

    GetIntVec($0B,old_isr);
    SetIntVec($0B,addr(serial_isr));

    old_int_mask := inport($21);    { enable interrupt on PIC }
    outport($21, old_int_mask AND $F7);

    asm
      sti
    end;
  end;


{ this function closes the port, turning off interrupts and }
{ restoring the old interrupt vector }
procedure close_serial;
  begin
    asm
      cli
    end;
    outport(ua[comport] + SER_MCR, 0); { disable the interrupts }
    outport(ua[comport] + SER_IER, 0);
    outport($21, old_int_mask);
    asm
      sti
    end;

    { reset old isr handler }
    SetIntVec($0B,old_isr);
  end;

{ Send the buffer }
procedure send_command;
  var
    i : integer;
    ch : char;
  begin
    i := 1;
    ch := out_buf[i];
    while (ch <> #00) do
      begin
        serial_write(byte(ch));
        i := i + 1;
        ch := out_buf[i];
      end;
  end;
{==================================================================}
procedure PICDefineChar(cn,d1,d2,d3,d4,d5,d6,d7,d8:byte);
  var
   k:boolean;
  begin
    picpushins(64+(cn shl 3));
    picpushchar(d1); picpushchar(d2); picpushchar(d3);
    picpushchar(d4); picpushchar(d5); picpushchar(d6);
    picpushchar(d7); picpushchar(d8);
  end;

procedure PICpushchar(ch:byte);
  begin
    serial_write(18);
    serial_write(ch);
  end;

procedure PICpushins(ch:byte);
  begin
    serial_write(17);
    serial_write(ch);
  end;

procedure PICClearLCD;
begin
  PICpushins(1);
  PICCursorOFF;
end;

procedure PICGotoXY(row,col:byte);
begin
  begin
    case row of
      1: picpushins(128+col-1);
      2: picpushins(128+col+63);
      3: picpushins(128+col+19);
      4: picpushins(128+col+83);
    end;
  end;
end;

procedure PICPrintLCD(msg:string);
begin
out_buf:=msg+#00;
send_command;
end;

procedure SetGP(gp1,gp2,gp3,gp4:byte);
{
this procedure will turn on any or all of the general purpose out lines
on the pic-an-lcd module.  gp1-4 correspond to  lines 1-4 on the module.
any value >0 will turn it on.  Set to 0's to turn them off again
}
var
gp:byte;
begin
gp:=0;
if gp1>0 then gp:=gp+1;
if gp2>0 then gp:=gp+2;
if gp3>0 then gp:=gp+4;
if gp4>0 then gp:=gp+8;
if gp=0 then gp:=128;
{out_buf:=#21+chr(gp)+#00;
send_command;}
serial_write(21);
serial_write(gp);
end;

 procedure PICCursorOn2;
   begin
     PICPushIns(13);
   end;

 procedure PICCursorOff2;
   begin
     PICPushIns(12);
   end;

procedure PICCursorOFF;
begin
  serial_write(25);
  serial_write(4);
end;

procedure PICCursorON;
begin
  serial_write(25);
  serial_write(1);
end;

procedure PICReadSettings(section:string);
begin
  Comport := ReadWord(section,'Comport');
end;

procedure PICClose;
 begin
  close_serial;        { close the connection }
 end;

procedure PICInit;
{ *** main *** }
begin
  serial_lock := 0;
  ser_end := -1;
  ser_start := -1;

{ Default baud rate = 9600 }
{ Initialize port handlers (open com) }
  open_serial;
  PICCursorOFF;
  PICClearLCD;
  SetGP(0,0,0,0); {set all gp lines off}
end;

procedure pictest;
begin

for i:=1 to 7 do
begin
PICClearLCD;
PICGotoXY(1,i);
PICPrintLCD('line 1 scroll');
delay(200);
end;

GotoXY(2,1);
PICPrintLCD('this is line 2');
PICCursorON;
  writeln('Connecting... (press <esc> to quit)');

  chb := ' ';
  while (chb<>chr(27)) do
    begin
      if KeyPressed then   { try and get a character from local machine }
        begin
          chb := ReadKey;
{if the modem/host doesn't return the char then you will need the next line }
          { write(chb); }
          serial_write(byte(chb));   { send the character to other machine }
         if chb=chr(13) then            { test for CR, if so add an line feed }
            begin
{if the modem/host doesn't return the char then you will need the next line }
              { writeln; }
              serial_write(10);
            end;
        end;

      ch1 := serial_read;  { try and get a character from remote }
      if (ch1 <= $FF) AND (ch1 >= 0)
        then write(char(ch1));
    end;
end;

begin

end.
